@@ -3,7 +3,7 @@ from sklearn.decomposition import PCA
 from sklearn.manifold import TSNE
 import matplotlib.pyplot as plt
 import seaborn as sns
-import umap.umap_ as umap
+import umap
 import os
 
 from ark.utils import misc_utils
