@@ -1,22 +1,98 @@
 import pandas as pd
 import xarray as xr
 import numpy as np
-from ark.utils import spatial_analysis_utils
-from ark.utils import misc_utils
+from tqdm.auto import tqdm
+from itertools import combinations_with_replacement
+from ark.utils import spatial_analysis_utils, misc_utils, load_utils, io_utils
 
 import ark.settings as settings
 
 
+def batch_channel_spatial_enrichment(label_dir, marker_thresholds, all_data, batch_size=5,
+                                     suffix='_feature_0', xr_channel_name='segmentation_label',
+                                     **kwargs):
+    """Wrapper function for batching calls to `calculate_channel_spatial_enrichment` over fovs
+
+    Args:
+        label_dir (str | Pathlike):
+            directory containing labeled tiffs
+        marker_thresholds (numpy.ndarray):
+            threshold values for positive marker expression
+        all_data (pandas.DataFrame):
+            data including fovs, cell labels, and cell expression matrix for all markers
+        batch_size (int):
+            fov count to load into memory at a time
+        suffix (str):
+            suffix for tif file names
+        xr_channel_name (str):
+            channel name for label data array
+        **kwargs (dict):
+            args passed to `calculate_channel_spatial_enrichment`
+
+    Returns:
+        tuple (list, xarray.DataArray):
+
+        - a list with each element consisting of a tuple of closenum and closenumrand for each
+          fov included in the analysis
+        - an xarray with dimensions (fovs, stats, num_channels, num_channels). The included
+          stats variables for each fov are z, muhat, sigmahat, p, h, adj_p, and
+          cluster_names
+    """
+
+    # parse files in label_dir
+    all_label_names = io_utils.list_files(label_dir, substrs=['.tif'])
+
+    included_fovs = kwargs.get('included_fovs', None)
+    if included_fovs:
+        label_fovs = io_utils.extract_delimited_names(all_label_names, delimiter=suffix)
+        all_label_names = \
+            [all_label_names[i] for i, fov in enumerate(label_fovs) if fov in included_fovs]
+
+    batching_strategy = \
+        [all_label_names[i:i + batch_size] for i in range(0, len(all_label_names), batch_size)]
+
+    # create containers for batched return values
+    values = []
+    stats_datasets = []
+
+    for batch_names in tqdm(batching_strategy, desc="Batch Completion", unit="batch"):
+        label_maps = load_utils.load_imgs_from_dir(label_dir, files=batch_names,
+                                                   xr_channel_names=[xr_channel_name],
+                                                   trim_suffix=suffix)
+
+        dist_mats = spatial_analysis_utils.calc_dist_matrix(label_maps)
+
+        # filter 'included_fovs'
+        if included_fovs:
+            filtered_includes = set(dist_mats.keys()).intersection(included_fovs)
+            kwargs['included_fovs'] = list(filtered_includes)
+
+        batch_vals, batch_stats = \
+            calculate_channel_spatial_enrichment(dist_mats, marker_thresholds, all_data, **kwargs)
+
+        # append new values
+        values = values + batch_vals
+
+        # append new data array (easier than iteratively combining)
+        stats_datasets.append(batch_stats)
+
+    # combine list of data arrays into one
+    stats = xr.concat(stats_datasets, dim="fovs")
+
+    return values, stats
+
+
 def calculate_channel_spatial_enrichment(dist_matrices_dict, marker_thresholds, all_data,
                                          excluded_channels=None, included_fovs=None,
                                          dist_lim=100, bootstrap_num=1000,
-                                         fov_col=settings.FOV_ID):
+                                         fov_col=settings.FOV_ID,
+                                         cell_label_col=settings.CELL_LABEL, context_col=None):
     """Spatial enrichment analysis to find significant interactions between cells expressing
     different markers. Uses bootstrapping to permute cell labels randomly.
 
     Args:
         dist_matrices_dict (dict):
-            Contains a cells x cells matrix with the euclidian distance between centers of
+            contains a cells x cells matrix with the euclidian distance between centers of
             corresponding cells for every fov
         marker_thresholds (numpy.ndarray):
             threshold values for positive marker expression
@@ -32,6 +108,10 @@ def calculate_channel_spatial_enrichment(dist_matrices_dict, marker_thresholds,
             number of permutations for bootstrap. Default is 1000.
         fov_col (str):
             column with the cell fovs.
+        cell_label_col (str):
+            cell label column name.
+        context_col (str):
+            column with context label.
 
     Returns:
         tuple (list, xarray.DataArray):
@@ -45,7 +125,7 @@ def calculate_channel_spatial_enrichment(dist_matrices_dict, marker_thresholds,
 
     # Setup input and parameters
     if included_fovs is None:
-        included_fovs = all_data[fov_col].unique()
+        included_fovs = list(dist_matrices_dict.keys())
         num_fovs = len(included_fovs)
     else:
         num_fovs = len(included_fovs)
@@ -88,6 +168,11 @@ def calculate_channel_spatial_enrichment(dist_matrices_dict, marker_thresholds,
     # Subsetting threshold matrix to only include column with threshold values
     thresh_vec = marker_thresholds.iloc[:, 1].values
 
+    # Only include the columns with the patient label, cell label, and cell phenotype
+    if context_col is not None:
+        context_names = all_data[context_col].unique()
+        context_pairings = combinations_with_replacement(context_names, 2)
+
     for fov in included_fovs:
         # Subsetting expression matrix to only include patients with correct fov label
         current_fov_idx = all_data[fov_col] == fov
@@ -100,13 +185,43 @@ def calculate_channel_spatial_enrichment(dist_matrices_dict, marker_thresholds,
         dist_matrix = dist_matrices_dict[fov]
 
         # Get close_num and close_num_rand
-        close_num, channel_nums, _ = spatial_analysis_utils.compute_close_cell_num(
+        close_num, channel_nums, mark_pos_labels = spatial_analysis_utils.compute_close_cell_num(
             dist_mat=dist_matrix, dist_lim=100, analysis_type="channel",
             current_fov_data=current_fov_data, current_fov_channel_data=current_fov_channel_data,
-            thresh_vec=thresh_vec)
+            thresh_vec=thresh_vec, cell_label_col=cell_label_col)
+
+        if context_col is not None:
+            close_num_rand = np.zeros((*close_num.shape, bootstrap_num), dtype=np.uint16)
+
+            context_nums_per_id = \
+                current_fov_data.groupby(context_col)[cell_label_col].apply(list).to_dict()
+
+            for name_i, name_j in context_pairings:
+                # some FoVs may not have cells with a certain context, so they are skipped here
+                try:
+                    context_cell_labels = context_nums_per_id[name_i]
+                    context_cell_labels.extend(context_nums_per_id[name_j])
+                except KeyError:
+                    continue
+                context_cell_labels = np.unique(context_cell_labels)
+
+                context_dist_mat = dist_matrix.loc[context_cell_labels, context_cell_labels]
+
+                context_pos_labels = [
+                    np.intersect1d(mark_pos_label, context_cell_labels)
+                    for mark_pos_label in mark_pos_labels
+                ]
+
+                context_mark_nums = [len(cpl) for cpl in context_pos_labels]
+
+                close_num_rand = close_num_rand + \
+                    spatial_analysis_utils.compute_close_cell_num_random(
+                        context_mark_nums, context_pos_labels, context_dist_mat, dist_lim,
+                        bootstrap_num
+                    )
 
         close_num_rand = spatial_analysis_utils.compute_close_cell_num_random(
-            channel_nums, dist_matrix, dist_lim, bootstrap_num)
+            channel_nums, mark_pos_labels, dist_matrix, dist_lim, bootstrap_num)
 
         values.append((close_num, close_num_rand))
 
@@ -116,11 +231,83 @@ def calculate_channel_spatial_enrichment(dist_matrices_dict, marker_thresholds,
     return values, stats
 
 
+def batch_cluster_spatial_enrichment(label_dir, all_data, batch_size=5, suffix='_feature_0',
+                                     xr_channel_name='segmentation_label', **kwargs):
+    """ Wrapper function for batching calls to `calculate_cluster_spatial_enrichment` over fovs
+
+    Args:
+        label_dir (str | Pathlike):
+            directory containing labeled tiffs
+        all_data (pandas.DataFrame):
+            data including fovs, cell labels, and cell expression matrix for all markers
+        batch_size (int):
+            fov count to load into memory at a time
+        suffix (str):
+            suffix for tif file names
+        xr_channel_name (str):
+            channel name for label data array
+        **kwargs (dict):
+            args passed to `calculate_cluster_spatial_enrichment`
+
+    Returns:
+        tuple (list, xarray.DataArray):
+
+        - a list with each element consisting of a tuple of closenum and closenumrand for each
+          fov included in the analysis
+        - an xarray with dimensions (fovs, stats, num_channels, num_channels). The included
+          stats variables for each fov are z, muhat, sigmahat, p, h, adj_p, and
+          cluster_names
+    """
+
+    # parse files in label_dir
+    all_label_names = io_utils.list_files(label_dir, substrs=['.tif'])
+
+    included_fovs = kwargs.get('included_fovs', None)
+    if included_fovs:
+        label_fovs = io_utils.extract_delimited_names(all_label_names, delimiter=suffix)
+        all_label_names = \
+            [all_label_names[i] for i, fov in enumerate(label_fovs) if fov in included_fovs]
+
+    batching_strategy = \
+        [all_label_names[i:i + batch_size] for i in range(0, len(all_label_names), batch_size)]
+
+    # create containers for batched return values
+    values = []
+    stats_datasets = []
+
+    for batch_names in tqdm(batching_strategy, desc="Batch Completion", unit="batch"):
+        label_maps = load_utils.load_imgs_from_dir(label_dir, files=batch_names,
+                                                   xr_channel_names=[xr_channel_name],
+                                                   trim_suffix=suffix)
+
+        dist_mats = spatial_analysis_utils.calc_dist_matrix(label_maps)
+
+        # filter 'included_fovs'
+        if included_fovs:
+            filtered_includes = set(dist_mats.keys()).intersection(included_fovs)
+            kwargs['included_fovs'] = list(filtered_includes)
+
+        batch_vals, batch_stats = \
+            calculate_cluster_spatial_enrichment(all_data, dist_mats, **kwargs)
+
+        # append new values
+        values = values + batch_vals
+
+        # append new data array (easier than iteratively combining)
+        stats_datasets.append(batch_stats)
+
+    # combine list of data arrays into one
+    stats = xr.concat(stats_datasets, dim="fovs")
+
+    return values, stats
+
+
 def calculate_cluster_spatial_enrichment(all_data, dist_matrices_dict, included_fovs=None,
                                          bootstrap_num=1000, dist_lim=100, fov_col=settings.FOV_ID,
                                          cluster_name_col=settings.CELL_TYPE,
                                          cluster_id_col=settings.CLUSTER_ID,
-                                         cell_label_col=settings.CELL_LABEL, context_labels=None):
+                                         cell_label_col=settings.CELL_LABEL, context_col=None,
+                                         distance_cols=None):
     """Spatial enrichment analysis based on cell phenotypes to find significant interactions
     between different cell types, looking for both positive and negative enrichment. Uses
     bootstrapping to permute cell labels randomly.
@@ -145,9 +332,10 @@ def calculate_cluster_spatial_enrichment(all_data, dist_matrices_dict, included_
             column with the cell phenotype IDs.
         cell_label_col (str):
             column with the cell labels.
-        context_labels (dict):
-            A dict that contains which specific types of cells we want to consider.
-            If argument is None, we will not run context-dependent spatial analysis
+        context_col (str):
+            column with context labels. If None, no context is assumed.
+        distance_cols (str):
+            column names of feature distances to include in analysis.
 
     Returns:
         tuple (list, xarray.DataArray):
@@ -161,7 +349,7 @@ def calculate_cluster_spatial_enrichment(all_data, dist_matrices_dict, included_
 
     # Setup input and parameters
     if included_fovs is None:
-        included_fovs = all_data[fov_col].unique()
+        included_fovs = list(dist_matrices_dict.keys())
         num_fovs = len(included_fovs)
     else:
         num_fovs = len(included_fovs)
@@ -172,6 +360,11 @@ def calculate_cluster_spatial_enrichment(all_data, dist_matrices_dict, included_
     misc_utils.verify_in_list(fov_names=included_fovs,
                               unique_fovs=all_data[fov_col].unique())
 
+    if distance_cols:
+        all_data, dist_matrices_dict = spatial_analysis_utils.append_distance_features_to_dataset(
+            dist_matrices_dict, all_data, distance_cols
+        )
+
     # Extract the names of the cell phenotypes
     cluster_names = all_data[cluster_name_col].drop_duplicates()
     # Extract the columns with the cell phenotype codes
@@ -180,7 +373,14 @@ def calculate_cluster_spatial_enrichment(all_data, dist_matrices_dict, included_
     cluster_num = len(cluster_ids)
 
     # Only include the columns with the patient label, cell label, and cell phenotype
-    all_pheno_data = all_data[[fov_col, cell_label_col, cluster_id_col]]
+    all_pheno_data_cols = [fov_col, cell_label_col, cluster_id_col]
+    all_pheno_data_cols += [] if context_col is None else [context_col]
+
+    all_pheno_data = all_data[all_pheno_data_cols]
+
+    if context_col is not None:
+        context_names = all_data[context_col].unique()
+        context_pairings = combinations_with_replacement(context_names, 2)
 
     # Create stats Xarray with the dimensions (fovs, stats variables, num_markers, num_markers)
     stats_raw_data = np.zeros((num_fovs, 7, cluster_num, cluster_num))
@@ -198,15 +398,46 @@ def calculate_cluster_spatial_enrichment(all_data, dist_matrices_dict, included_
         dist_mat = dist_matrices_dict[fov]
 
         # Get close_num and close_num_rand
-        close_num, pheno_nums, pheno_nums_per_id = spatial_analysis_utils.compute_close_cell_num(
+        close_num, pheno_nums, mark_pos_labels = spatial_analysis_utils.compute_close_cell_num(
             dist_mat=dist_mat, dist_lim=dist_lim, analysis_type="cluster",
-            current_fov_data=current_fov_pheno_data, cluster_ids=cluster_ids)
+            current_fov_data=current_fov_pheno_data, cluster_ids=cluster_ids,
+            cell_label_col=cell_label_col)
 
-        close_num_rand = spatial_analysis_utils.compute_close_cell_num_random(
-            pheno_nums, dist_mat, dist_lim, bootstrap_num)
+        # subset distance matrix with context
+        if context_col is not None:
+            close_num_rand = np.zeros((*close_num.shape, bootstrap_num), dtype=np.uint16)
 
-        # close_num_rand_context = spatial_analysis_utils.compute_close_cell_num_random(
-        #     pheno_nums_per_id, dist_mat, dist_lim, bootstrap_num)
+            context_nums_per_id = \
+                current_fov_pheno_data.groupby(context_col)[cell_label_col].apply(list).to_dict()
+
+            for name_i, name_j in context_pairings:
+                # some FoVs may not have cells with a certain context, so they are skipped here
+                try:
+                    context_cell_labels = context_nums_per_id[name_i]
+                    context_cell_labels.extend(context_nums_per_id[name_j])
+                except KeyError:
+                    continue
+
+                context_cell_labels = np.unique(context_cell_labels)
+
+                context_dist_mat = dist_mat.loc[context_cell_labels, context_cell_labels]
+
+                context_pos_labels = [
+                    np.intersect1d(mark_pos_label, context_cell_labels)
+                    for mark_pos_label in mark_pos_labels
+                ]
+
+                context_pheno_nums = [len(cpl) for cpl in context_pos_labels]
+
+                close_num_rand = close_num_rand + \
+                    spatial_analysis_utils.compute_close_cell_num_random(
+                        context_pheno_nums, context_pos_labels, context_dist_mat, dist_lim,
+                        bootstrap_num
+                    )
+
+        else:
+            close_num_rand = spatial_analysis_utils.compute_close_cell_num_random(
+                pheno_nums, mark_pos_labels, dist_mat, dist_lim, bootstrap_num)
 
         values.append((close_num, close_num_rand))
 
