@@ -11,6 +11,7 @@ from scipy.spatial.distance import cdist
 
 import ark.settings as settings
 from ark.utils import io_utils, misc_utils
+from ark.utils._bootstrapping import compute_close_num_rand
 
 
 def calc_dist_matrix(label_maps, save_path=None):
@@ -62,6 +63,52 @@ def calc_dist_matrix(label_maps, save_path=None):
         np.savez(os.path.join(save_path, "dist_matrices.npz"), **dist_matrices)
 
 
+def append_distance_features_to_dataset(dist_mats, cell_table, distance_columns):
+    """Appends selected distance features as 'cells' in distance matrix and cell table
+
+    Args:
+        dist_mats (dict):
+            Distance matricies as output by `calc_dist_matrix`, indexed by fov name
+        cell_table (pd.DataFrame):
+            Table of cell features. Must contain provided distance columns
+        distance_columns (List[str]):
+            List of column names which store feature distance.  These must exist in cell_table
+
+    Returns:
+        (pd.DataFrame, dict):
+            Updated cell_table, and distance matricie indexed by fov name
+    """
+
+    misc_utils.verify_in_list(distance_columns=distance_columns, valid_columns=cell_table.columns)
+
+    num_cell_types = max(cell_table[settings.CLUSTER_ID].unique())
+
+    for fov in dist_mats.keys():
+        fov_cells = cell_table.loc[cell_table[settings.FOV_ID] == fov]
+        num_labels = max(fov_cells[settings.CELL_LABEL])
+        for i, dist_col in enumerate(distance_columns):
+            cell_table = cell_table.append(pd.DataFrame([{
+                settings.FOV_ID: fov,
+                settings.CELL_LABEL: num_labels + i + 1,
+                settings.CELL_TYPE: dist_col,
+                settings.CLUSTER_ID: num_cell_types + i + 1,
+            }]))
+            coords = (
+                [max(dist_mats[fov].dim_0.values) + i + 1],
+                fov_cells[settings.CELL_LABEL].values[:]
+            )
+
+            dist_mats[fov] = xr.concat([dist_mats[fov], xr.DataArray(
+                fov_cells[dist_col].values[np.newaxis, :], coords=coords
+            )], dim='dim_0')
+
+            dist_mats[fov] = xr.concat([dist_mats[fov], xr.DataArray(
+                fov_cells[dist_col].values[:, np.newaxis], coords=(coords[1], coords[0])
+            )], dim='dim_1')
+
+    return cell_table, dist_mats
+
+
 def get_pos_cell_labels_channel(thresh, current_fov_channel_data, cell_labels, current_marker):
     """For channel enrichment, finds positive labels that match the current phenotype
     or identifies cells with positive expression values for the current marker
@@ -185,10 +232,9 @@ def compute_close_cell_num(dist_mat, dist_lim, analysis_type,
     mark1poslabels = []
 
     dist_mat_bin = xr.DataArray(
-        (dist_mat.values < dist_lim).astype(np.uint8),
+        ((dist_mat.values < dist_lim) & (dist_mat.values > 0)).astype(np.uint8),
         coords=dist_mat.coords
     )
-
     for j in range(num):
         if analysis_type == "cluster":
             mark1poslabels.append(
@@ -204,13 +250,6 @@ def compute_close_cell_num(dist_mat, dist_lim, analysis_type,
                                             current_marker=current_fov_channel_data.columns[j]))
         mark1_num.append(len(mark1poslabels[j]))
 
-    # we'll need this because for cluster-based context-dependent randomization
-    # we need to facet our randomization of labels based on the cell_types and associated
-    # cell_ids the user specifies
-    mark1labels_per_id = None
-    if analysis_type == "cluster":
-        mark1labels_per_id = dict(zip(cluster_ids, mark1poslabels))
-
     # iterating k from [j, end] cuts out 1/2 the steps (while symmetric)
     for j, m1n in enumerate(mark1_num):
         for k, m2n in enumerate(mark1_num[j:], j):
@@ -224,18 +263,23 @@ def compute_close_cell_num(dist_mat, dist_lim, analysis_type,
             # symmetry :)
             close_num[k, j] = close_num[j, k]
 
-    return close_num, mark1_num, mark1labels_per_id
+    return close_num, mark1_num, mark1poslabels
 
 
-def compute_close_cell_num_random(marker_nums, dist_mat, dist_lim, bootstrap_num):
+# TODO: passing marker_nums and mark_pos_labels is redundant:
+#       marker_nums[j] = len(mark_pos_labels[j])
+def compute_close_cell_num_random(marker_nums, mark_pos_labels, dist_mat, dist_lim, bootstrap_num):
     """Uses bootstrapping to permute cell labels randomly and records the number of close cells
-    (within the dit_lim) in that random setup.
+    (within the dist_lim) in that random setup.
 
     Args:
         marker_nums (numpy.ndarray):
             list of cell counts of each marker type
-        dist_mat (numpy.ndarray):
-            cells x cells matrix with the euclidian distance between centers of corresponding cells
+        mark_pos_labels (list):
+            cell labels for each marker number
+        dist_mat (xr.DataArray):
+            cells x cells matrix with the euclidian distance between centers of corresponding
+            cells. This can be indexed by cell label
         dist_lim (int):
             threshold for spatial enrichment distance proximity
         bootstrap_num (int):
@@ -246,23 +290,50 @@ def compute_close_cell_num_random(marker_nums, dist_mat, dist_lim, bootstrap_num
             Large matrix of random positive marker counts for every permutation in the bootstrap
     """
 
-    # Create close_num_rand
-    close_num_rand = np.zeros((
-        len(marker_nums), len(marker_nums), bootstrap_num), dtype=np.uint16)
+    # Generate binarized distance matrix
+    dist_mat_bin = ((dist_mat.values < dist_lim) & (dist_mat.values > 0)).astype(np.uint16)
 
-    dist_mat_bin_flattened = (dist_mat.values < dist_lim).astype(np.uint8).flatten()
+    # assures that marker counts don't exceed number of cells
+    for mn in marker_nums:
+        if mn > dist_mat_bin.shape[0]:
+            raise ValueError('Marker number count can not be greater than number of cells...')
 
-    for j, m1n in enumerate(marker_nums):
-        for k, m2n in enumerate(marker_nums[j:], j):
-            samples_dim = (m1n * m2n, bootstrap_num)
-            count_close_num_rand_hits = np.sum(
-                np.random.choice(dist_mat_bin_flattened, samples_dim, True), axis=0,
-                dtype=np.uint16
-            )
+    # flattens list-of-list representation into 1D array and stores the index keys
+    row_indicies = [0]
+    cols_in_row_flat = []
 
-            close_num_rand[j, k, :] = count_close_num_rand_hits
-            # symmetry :)
-            close_num_rand[k, j, :] = close_num_rand[j, k, :]
+    for row in range(dist_mat_bin.shape[0]):
+        cols = list(np.nonzero(dist_mat_bin[row, :])[0])
+        cols_in_row_flat.extend(cols)
+        row_indicies.append(len(cols) + row_indicies[row])
+
+    # formats list-of-list representation into cython compatable argument
+    cols_in_row_flat = np.array(cols_in_row_flat, dtype=np.uint16)
+    _row_indicies = np.array(row_indicies, dtype=np.uint64)
+
+    # sort marker_nums and save permutation
+    # this can speed up compute_close_num_rand
+    marker_order = [
+        (
+            mn,
+            np.flatnonzero(dist_mat[dist_mat.dims[0]].isin(mark_pos_labels[i])).astype(np.uint64),
+            i
+        )
+        for i, mn in enumerate(marker_nums)
+    ]
+    marker_order.sort(key=lambda x: x[0])
+    sorted_marker_nums, sorted_pos_labels, sort_permutation = zip(*marker_order)
+    _marker_nums = np.array(sorted_marker_nums, dtype=np.uint16)
+    _pos_labels = {i: v for i, v in enumerate(sorted_pos_labels)}
+
+    # performing bootstrapping
+    close_num_rand = compute_close_num_rand(dist_mat_bin, cols_in_row_flat, _row_indicies,
+                                            _marker_nums, _pos_labels, int(bootstrap_num))
+
+    # unpermute close_num_rand
+    x_scramble = np.tile(np.argsort(sort_permutation), (len(sort_permutation), 1))
+    y_scramble = x_scramble.T
+    close_num_rand = close_num_rand[x_scramble, y_scramble, :]
 
     return close_num_rand
 
